/*
 * file: PvEngine.c
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#include <stdio.h>

#include <pv_register.h>

#include "PvEngine.h"
#include "PvEngineAttr.h"
#include "Pv.h"


PyObject *PvEngine_new(PyTypeObject *type, PyObject *args)
{
	PvEngineAttrObject *pv_attr;

	PvEngineObject *self;
	int ret;

	//printf("DEBUG: %s\n", __FUNCTION__);
	if (!PyArg_ParseTuple(args, "O!", &PvEngineAttr_Type, &pv_attr))
		return NULL;

	self = (PvEngineObject*)type->tp_alloc(type, 0);
	if (!self)
		return NULL;

	if ((ret = pv_init(&self->engine, &pv_attr->attr)) != 0) {
		PyErr_SetString(PyExc_AttributeError, "pv_init failed");
		Py_DECREF(self);
		return NULL;
	}

	return (PyObject *)self;
}

PyObject *PvEngine_register(PvEngineObject *self, PyObject *args)
{
	char *name;
	struct pv *var;
	PvObject *pv_obj;

	//printf("DEBUG: %s\n", __FUNCTION__);
	if (!PyArg_ParseTuple(args, "s", &name))
		return NULL;

	if ((var = pv_register_lookup((struct pv_engine *)&self->engine, name)) != 0) {
		pv_obj = PyObject_New(PvObject, &Pv_Type);
		pv_obj->engine = (struct pv_engine*)&self->engine;
		pv_obj->pv = var;
		return (PyObject *)pv_obj;
	}

	Py_INCREF(Py_None);
	return Py_None;
}

PyObject *PvEngine_lock(PvEngineObject *self, PyObject *args)
{
	//printf("DEBUG: %s\n", __FUNCTION__);

	pv_lock(&self->engine);

	Py_INCREF(Py_None);
	return Py_None;
}

PyObject *PvEngine_unlock(PvEngineObject *self, PyObject *args)
{
	//printf("DEBUG: %s\n", __FUNCTION__);

	pv_unlock(&self->engine);

	Py_INCREF(Py_None);
	return Py_None;
}

void PvEngine_dealloc(PvEngineObject *self)
{
	//printf("DEBUG: %s\n", __FUNCTION__);
	int ret;
	if ((ret = pv_destroy(&self->engine)) != 0)
		PyErr_SetString(PyExc_RuntimeError, "Failed to destroy the pv_engine");

	PyObject_Del(self);
}

PyMethodDef PvEngine_methods[] = {
	{"register",	(PyCFunction)PvEngine_register,	METH_VARARGS,	PyDoc_STR("register...")},
	{"lock",	(PyCFunction)PvEngine_lock,	METH_VARARGS,	PyDoc_STR("lock...")},
	{"unlock",	(PyCFunction)PvEngine_unlock,	METH_VARARGS,	PyDoc_STR("unlock...")},
	{NULL},
};

PyTypeObject PvEngine_Type = {
	/* The ob_type field must be initialized in the module init function
	 * to be portable to Windows without using C++. */
	PyObject_HEAD_INIT(NULL)	/*ob_type*/
	0,				/*ob_size*/
	"libpv.PvEngine",		/*tp_name*/
	sizeof(PvEngineObject),		/*tp_basicsize*/
	0,				/*tp_itemsize*/
	/* methods */
	(destructor)PvEngine_dealloc,	/*tp_dealloc*/
	0,				/*tp_print*/
	0,				/*tp_getattr*/
	0,				/*tp_setattr*/
	0,				/*tp_compare*/
	0,				/*tp_repr*/
	0,				/*tp_as_number*/
	0,				/*tp_as_sequence*/
	0,				/*tp_as_mapping*/
	0,				/*tp_hash*/
	0,				/*tp_call*/
	0,				/*tp_str*/
	0,				/*tp_getattro*/
	0,				/*tp_setattro*/
	0,				/*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT,		/*tp_flags*/
	0,				/*tp_doc*/
	0,				/*tp_traverse*/
	0,				/*tp_clear*/
	0,				/*tp_richcompare*/
	0,				/*tp_weaklistoffset*/
	0,				/*tp_iter*/
	0,				/*tp_iternext*/
	PvEngine_methods,		/*tp_methods*/
	0,				/*tp_members*/
	0,				/*tp_getset*/
	0,				/*tp_base*/
	0,				/*tp_dict*/
	0,				/*tp_descr_get*/
	0,				/*tp_descr_set*/
	0,				/*tp_dictoffset*/
	0,				/*tp_init*/
	0,				/*tp_alloc*/
	(newfunc)PvEngine_new,		/*tp_new*/
	0,				/*tp_free*/
	0,				/*tp_is_gc*/
};

