/*
 * file: pv_destroy.c
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#include <libpv_config.h>

#include <errno.h>
#include <stdlib.h>
#include <unistd.h>

#include <pv_destroy.h>
#include <pv_lock.h>
#include <pv_bs.h>
#include <pv_jvisu.h>
#include <pv_ev.h>		/* only used, when event handling is enabled */

int pv_destroy_free_pvs(ptxlist	*pv_list)
{
	int finished = 0;

	if (!pv_list)
		return -EINVAL;

	for (goto_head(&pv_list); !finished; goto_next(&pv_list)) {

		struct pv *pv = (struct pv *)list_get_element_data(pv_list);

		if (is_last(pv_list))
			finished=1;

		if (!pv) {
			dprintf(LEVEL_ERROR, "(%s) error: trying to free nonexisting list element\n",
				__FUNCTION__);
			continue;
		}

		if (pv->device_file != -1) {
			close(pv->device_file);
			pv->device_file = -1;
		}

		free(pv->id);
		free(pv->name);
		free(pv->device);

		if (pv->type == PV_TYPE_STRING) {
			free(pv->dflt.string);
			free(pv->max.string);
			free(pv->min.string);
		}
	}

	list_del(pv_list);

	return 0;
}

/**
 * pv_destroy - Clean up all used resources in this instance
 * @engine_anon: Current process image to work for
 *
 * Description:
 *
 * Return value:
 **/
int pv_destroy (pv_engine_t	*engine_anon)
{
	struct pv_engine	*engine = (struct pv_engine *)engine_anon;
	int			err = 0, ret;

	if (!engine_anon)
		return -EINVAL;

	dprintf(LEVEL_INFO, "(%s) destroying pv_engine\n", __FUNCTION__);

	ret = pv_jvisu_destroy(engine);
	if (ret) {
		err = ret;
		dprintf(LEVEL_ERROR, "(%s) failed to destroy the jvisu server stuff, err %d\n",
			__PRETTY_FUNCTION__, err);
	}

	ret = pv_lock_destroy(engine);
	if (ret) {
		err = ret;
		dprintf(LEVEL_ERROR, "(%s) failed to destroy lock, err %d\n",
			__FUNCTION__, err);
	}

	/* init event handling for this instance */
	pv_event_exit_handling(engine);

	ret = pv_bs_destroy(engine);
	if (ret) {
		err = ret;
		dprintf(LEVEL_ERROR, "(%s) failed to destroy backing store, err %d\n",
			__FUNCTION__, err);
	}

	/* remove process variables */
	ret = pv_destroy_free_pvs(engine->pv_list);
	if (ret) {
		err = ret;
		dprintf(LEVEL_ERROR, "(%s) failed to destroy pv_list, err %d\n",
			__FUNCTION__, err);
	}

	return err;
}

/* end of file pv_destroy.c */
